package org.eclipse.papyrusrt.bb.bbif.papyrus.trans.to.uml

import java.util.Map

import java.util.List
import org.eclipse.emf.ecore.EObject
import org.eclipse.papyrusrt.bb.bbIF.BBDataType
import org.eclipse.papyrusrt.bb.bbIF.BBIF
import org.eclipse.papyrusrt.bb.bbif.BBIFProfile.Direction
import org.eclipse.papyrusrt.bb.bbIF.BBModel
import org.eclipse.papyrusrt.bb.bbIF.BBProperty
import org.eclipse.papyrusrt.bb.bbIF.BBProtocol
import org.eclipse.papyrusrt.bb.bbIF.BBSignal
import org.eclipse.papyrusrt.bb.bbIF.SubSystem
import org.eclipse.papyrusrt.xtumlrt.common.Attribute
import org.eclipse.papyrusrt.xtumlrt.common.CommonFactory
import org.eclipse.papyrusrt.xtumlrt.common.NamedElement
import org.eclipse.papyrusrt.xtumlrt.common.Package
import org.eclipse.papyrusrt.xtumlrt.common.Protocol
import org.eclipse.papyrusrt.xtumlrt.common.ProtocolBehaviourFeatureKind
import org.eclipse.papyrusrt.xtumlrt.common.Signal
import org.eclipse.papyrusrt.xtumlrt.trans.to.uml.umlrt.Xtumlrt2UMLTranslator
import org.eclipse.papyrusrt.xtumlrt.umlrt.RTModel
import org.eclipse.papyrusrt.xtumlrt.umlrt.RTPassiveClass
import org.eclipse.papyrusrt.xtumlrt.util.XTUMLRTLogger
import org.eclipse.uml2.uml.PackageableElement
import static extension org.eclipse.papyrusrt.umlrt.core.utils.ProtocolContainerUtils.*
import static extension org.eclipse.papyrusrt.xtumlrt.util.GeneralUtil.*

class BBIF2UMLTranslator extends Xtumlrt2UMLTranslator
{

	override generate(RTModel model)
	{
		XTUMLRTLogger.info("Invoking BBIF2UMLTranslator")
		super.generate(model)
	}

	override protected def create umlElement: super._transform(preprocess(element) as RTModel)
	_transform(RTModel element)
	{
		extendedTransform(element as RTModel,
			umlElement as org.eclipse.uml2.uml.Model)
	}

	override protected def create umlElement: super._transform(preprocess(element) as Package)
	_transform(Package element)
	{
		extendedTransform(element as Package,
			umlElement as org.eclipse.uml2.uml.Package)
	}

	override protected def create umlElement: super._transform(preprocess(element) as Protocol)
	_transform(Protocol element)
	{
		extendedTransform(element as Protocol,
			umlElement as org.eclipse.uml2.uml.Collaboration)
	}

	override protected def create umlElement: super._transform(preprocess(element) as Signal)
	_transform(Signal element)
	{
		extendedTransform(element as Signal,
			umlElement as org.eclipse.uml2.uml.CallEvent)
	}

	override protected def create umlElement: super._transform(preprocess(element) as Attribute)
	_transform(Attribute element)
	{
		extendedTransform(element as Attribute,
			umlElement as org.eclipse.uml2.uml.Property)
	}

	override protected def create umlElement: super._transform(preprocess(element) as RTPassiveClass)
	_transform(RTPassiveClass element)
	{
		extendedTransform(element as RTPassiveClass,
			umlElement as org.eclipse.uml2.uml.Class)
	}

	protected dispatch def preprocess(BBModel element)
	{
		element
	}

	protected dispatch def preprocess(SubSystem element)
	{
		addAnnotation(element, "SubSystem", #{"platform" -> element.platform})
		element
	}

	protected dispatch def preprocess(BBIF element)
	{
		addAnnotation(element, "BBIF", #{})
		element
	}

	protected dispatch def preprocess(BBProtocol element)
	{
		addAnnotation(element, "BBProtocol", #{})
		element
	}

	protected dispatch def preprocess(BBSignal element)
	{
		addAnnotation(element, "BBSignal", #{"direction" -> element.direction})
		element
	}

	protected dispatch def preprocess(BBDataType element)
	{
		addAnnotation(element, "BBDataType", #{})
		element
	}

	protected dispatch def preprocess(BBProperty element)
	{
		addAnnotation(element, "BBProperty",
			#{"bigEndian" -> element.bigEndian})
		element
	}

	protected def addAnnotation(NamedElement element, String annotationString,
		Map<String, Object> parameters)
	{
		val newAnnotation = CommonFactory.eINSTANCE.createAnnotation => [
			name = annotationString
		]
		parameters.forEach [ key, paramval |
			newAnnotation.parameters.add(
				CommonFactory.eINSTANCE.createAnnotationParameter => [
					name = key
					value = transformBBStereotypePropertyValue(paramval)
				])
		]
		element.annotations.add(newAnnotation)
		element
	}

	protected dispatch def extendedTransform(NamedElement element,
		org.eclipse.uml2.uml.NamedElement umlElement)
	{}

	protected dispatch def extendedTransform(BBModel element,
		org.eclipse.uml2.uml.Package umlElement)
	{
		val transformedTopSubsystems = element?.topSubsystems.transformList(
			PackageableElement)
		umlElement?.packagedElements.addAllNonPresent(
			transformedTopSubsystems)
	}

	protected dispatch def extendedTransform(SubSystem element,
		org.eclipse.uml2.uml.Package umlElement)
	{
		val transformedSubsystems = element?.subsystems.transformList(
			PackageableElement)
		val transformedBBIFs = element?.bbif.transformList(PackageableElement)
		umlElement?.packagedElements.addAllNonPresent(transformedSubsystems)
		umlElement?.packagedElements.addAllNonPresent(transformedBBIFs)
	}

	protected dispatch def extendedTransform(BBIF element,
		org.eclipse.uml2.uml.Package umlElement)
	{
		val transformedProtocols = element?.bbprotocols.transformList(
			PackageableElement).map[getProtocolContainer]
		val transformedDataTypes = element?.bbdatatypes.transformList(
			PackageableElement)
		umlElement?.packagedElements.addAllNonPresent(transformedProtocols)
		umlElement?.packagedElements.addAllNonPresent(transformedDataTypes)
	}

	protected dispatch def extendedTransform(BBProtocol element,
		org.eclipse.uml2.uml.Collaboration umlElement)
	{
		val transformedSignals = element?.bbsignals.transformList(
			PackageableElement)
//		umlElement?.packagedElements.addAllNonPresent(transformedSignals)
	}

	protected dispatch def extendedTransform(BBSignal element,
		org.eclipse.uml2.uml.CallEvent umlElement)
	{
	}

	protected dispatch def extendedTransform(BBDataType element,
		org.eclipse.uml2.uml.Class umlElement)
	{
		val transformedProperties = element?.bbproperties.transformList(
			org.eclipse.uml2.uml.Property)
		umlElement?.ownedAttributes.addAllNonPresent(transformedProperties)
	}

	protected dispatch def extendedTransform(BBProperty element,
		org.eclipse.uml2.uml.Package umlElement)
	{
	}

	protected dispatch def transformBBStereotypePropertyValue(
		ProtocolBehaviourFeatureKind kind)
	{
		switch (kind)
		{
			case IN: Direction.IN
			case OUT: Direction.OUT
			case INOUT: Direction.INOUT
		}
	}

	protected dispatch def transformBBStereotypePropertyValue(Object value)
	{
		value
	}

	protected def <T> transformList(List<? extends EObject> sourceList,
		Class<T> type)
	{
		sourceList.map[transform].filter(type).toList
	}

}
