package org.eclipse.papyrusrt.bb.bbif.papyrus.trans.to.uml;

import com.google.common.collect.Iterables;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.BiConsumer;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.papyrusrt.bb.bbIF.BBDataType;
import org.eclipse.papyrusrt.bb.bbIF.BBIF;
import org.eclipse.papyrusrt.bb.bbIF.BBModel;
import org.eclipse.papyrusrt.bb.bbIF.BBProperty;
import org.eclipse.papyrusrt.bb.bbIF.BBProtocol;
import org.eclipse.papyrusrt.bb.bbIF.BBSignal;
import org.eclipse.papyrusrt.bb.bbIF.SubSystem;
import org.eclipse.papyrusrt.bb.bbif.BBIFProfile.Direction;
import org.eclipse.papyrusrt.umlrt.core.utils.ProtocolContainerUtils;
import org.eclipse.papyrusrt.xtumlrt.common.Annotation;
import org.eclipse.papyrusrt.xtumlrt.common.AnnotationParameter;
import org.eclipse.papyrusrt.xtumlrt.common.Attribute;
import org.eclipse.papyrusrt.xtumlrt.common.CommonFactory;
import org.eclipse.papyrusrt.xtumlrt.common.NamedElement;
import org.eclipse.papyrusrt.xtumlrt.common.Protocol;
import org.eclipse.papyrusrt.xtumlrt.common.ProtocolBehaviourFeatureKind;
import org.eclipse.papyrusrt.xtumlrt.common.Signal;
import org.eclipse.papyrusrt.xtumlrt.trans.to.uml.umlrt.Xtumlrt2UMLTranslator;
import org.eclipse.papyrusrt.xtumlrt.umlrt.RTModel;
import org.eclipse.papyrusrt.xtumlrt.umlrt.RTPassiveClass;
import org.eclipse.papyrusrt.xtumlrt.util.GeneralUtil;
import org.eclipse.papyrusrt.xtumlrt.util.XTUMLRTLogger;
import org.eclipse.uml2.uml.CallEvent;
import org.eclipse.uml2.uml.Collaboration;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.PackageableElement;
import org.eclipse.uml2.uml.Property;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ListExtensions;
import org.eclipse.xtext.xbase.lib.ObjectExtensions;
import org.eclipse.xtext.xbase.lib.Pair;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

@SuppressWarnings("all")
public class BBIF2UMLTranslator extends Xtumlrt2UMLTranslator {
  @Override
  public int generate(final RTModel model) {
    int _xblockexpression = (int) 0;
    {
      XTUMLRTLogger.info("Invoking BBIF2UMLTranslator");
      _xblockexpression = super.generate(model);
    }
    return _xblockexpression;
  }
  
  @Override
  protected EObject _transform(final RTModel element) {
    final ArrayList<?> _cacheKey = CollectionLiterals.newArrayList(element);
    final EObject _result;
    synchronized (_createCache__transform) {
      if (_createCache__transform.containsKey(_cacheKey)) {
        return _createCache__transform.get(_cacheKey);
      }
      NamedElement _preprocess = this.preprocess(element);
      EObject __transform = super._transform(((RTModel) _preprocess));
      _result = __transform;
      _createCache__transform.put(_cacheKey, _result);
    }
    _init__transform(_result, element);
    return _result;
  }
  
  private final HashMap<ArrayList<?>, EObject> _createCache__transform = CollectionLiterals.newHashMap();
  
  private void _init__transform(final EObject umlElement, final RTModel element) {
    this.extendedTransform(((RTModel) element), 
      ((Model) umlElement));
  }
  
  @Override
  protected EObject _transform(final org.eclipse.papyrusrt.xtumlrt.common.Package element) {
    final ArrayList<?> _cacheKey = CollectionLiterals.newArrayList(element);
    final EObject _result;
    synchronized (_createCache__transform_1) {
      if (_createCache__transform_1.containsKey(_cacheKey)) {
        return _createCache__transform_1.get(_cacheKey);
      }
      NamedElement _preprocess = this.preprocess(element);
      EObject __transform = super._transform(((org.eclipse.papyrusrt.xtumlrt.common.Package) _preprocess));
      _result = __transform;
      _createCache__transform_1.put(_cacheKey, _result);
    }
    _init__transform_1(_result, element);
    return _result;
  }
  
  private final HashMap<ArrayList<?>, EObject> _createCache__transform_1 = CollectionLiterals.newHashMap();
  
  private void _init__transform_1(final EObject umlElement, final org.eclipse.papyrusrt.xtumlrt.common.Package element) {
    this.extendedTransform(((org.eclipse.papyrusrt.xtumlrt.common.Package) element), 
      ((org.eclipse.uml2.uml.Package) umlElement));
  }
  
  @Override
  protected EObject _transform(final Protocol element) {
    final ArrayList<?> _cacheKey = CollectionLiterals.newArrayList(element);
    final EObject _result;
    synchronized (_createCache__transform_2) {
      if (_createCache__transform_2.containsKey(_cacheKey)) {
        return _createCache__transform_2.get(_cacheKey);
      }
      NamedElement _preprocess = this.preprocess(element);
      EObject __transform = super._transform(((Protocol) _preprocess));
      _result = __transform;
      _createCache__transform_2.put(_cacheKey, _result);
    }
    _init__transform_2(_result, element);
    return _result;
  }
  
  private final HashMap<ArrayList<?>, EObject> _createCache__transform_2 = CollectionLiterals.newHashMap();
  
  private void _init__transform_2(final EObject umlElement, final Protocol element) {
    this.extendedTransform(((Protocol) element), 
      ((Collaboration) umlElement));
  }
  
  @Override
  protected EObject _transform(final Signal element) {
    final ArrayList<?> _cacheKey = CollectionLiterals.newArrayList(element);
    final EObject _result;
    synchronized (_createCache__transform_3) {
      if (_createCache__transform_3.containsKey(_cacheKey)) {
        return _createCache__transform_3.get(_cacheKey);
      }
      NamedElement _preprocess = this.preprocess(element);
      EObject __transform = super._transform(((Signal) _preprocess));
      _result = __transform;
      _createCache__transform_3.put(_cacheKey, _result);
    }
    _init__transform_3(_result, element);
    return _result;
  }
  
  private final HashMap<ArrayList<?>, EObject> _createCache__transform_3 = CollectionLiterals.newHashMap();
  
  private void _init__transform_3(final EObject umlElement, final Signal element) {
    this.extendedTransform(((Signal) element), 
      ((CallEvent) umlElement));
  }
  
  @Override
  protected EObject _transform(final Attribute element) {
    final ArrayList<?> _cacheKey = CollectionLiterals.newArrayList(element);
    final EObject _result;
    synchronized (_createCache__transform_4) {
      if (_createCache__transform_4.containsKey(_cacheKey)) {
        return _createCache__transform_4.get(_cacheKey);
      }
      NamedElement _preprocess = this.preprocess(element);
      EObject __transform = super._transform(((Attribute) _preprocess));
      _result = __transform;
      _createCache__transform_4.put(_cacheKey, _result);
    }
    _init__transform_4(_result, element);
    return _result;
  }
  
  private final HashMap<ArrayList<?>, EObject> _createCache__transform_4 = CollectionLiterals.newHashMap();
  
  private void _init__transform_4(final EObject umlElement, final Attribute element) {
    this.extendedTransform(((Attribute) element), 
      ((Property) umlElement));
  }
  
  @Override
  protected EObject _transform(final RTPassiveClass element) {
    final ArrayList<?> _cacheKey = CollectionLiterals.newArrayList(element);
    final EObject _result;
    synchronized (_createCache__transform_5) {
      if (_createCache__transform_5.containsKey(_cacheKey)) {
        return _createCache__transform_5.get(_cacheKey);
      }
      NamedElement _preprocess = this.preprocess(element);
      EObject __transform = super._transform(((RTPassiveClass) _preprocess));
      _result = __transform;
      _createCache__transform_5.put(_cacheKey, _result);
    }
    _init__transform_5(_result, element);
    return _result;
  }
  
  private final HashMap<ArrayList<?>, EObject> _createCache__transform_5 = CollectionLiterals.newHashMap();
  
  private void _init__transform_5(final EObject umlElement, final RTPassiveClass element) {
    this.extendedTransform(((RTPassiveClass) element), 
      ((org.eclipse.uml2.uml.Class) umlElement));
  }
  
  protected NamedElement _preprocess(final BBModel element) {
    return element;
  }
  
  protected NamedElement _preprocess(final SubSystem element) {
    SubSystem _xblockexpression = null;
    {
      String _platform = element.getPlatform();
      Pair<String, String> _mappedTo = Pair.<String, String>of("platform", _platform);
      this.addAnnotation(element, "SubSystem", Collections.<String, Object>unmodifiableMap(CollectionLiterals.<String, Object>newHashMap(_mappedTo)));
      _xblockexpression = element;
    }
    return _xblockexpression;
  }
  
  protected NamedElement _preprocess(final BBIF element) {
    BBIF _xblockexpression = null;
    {
      this.addAnnotation(element, "BBIF", Collections.<String, Object>unmodifiableMap(CollectionLiterals.<String, Object>newHashMap()));
      _xblockexpression = element;
    }
    return _xblockexpression;
  }
  
  protected NamedElement _preprocess(final BBProtocol element) {
    BBProtocol _xblockexpression = null;
    {
      this.addAnnotation(element, "BBProtocol", Collections.<String, Object>unmodifiableMap(CollectionLiterals.<String, Object>newHashMap()));
      _xblockexpression = element;
    }
    return _xblockexpression;
  }
  
  protected NamedElement _preprocess(final BBSignal element) {
    BBSignal _xblockexpression = null;
    {
      ProtocolBehaviourFeatureKind _direction = element.getDirection();
      Pair<String, ProtocolBehaviourFeatureKind> _mappedTo = Pair.<String, ProtocolBehaviourFeatureKind>of("direction", _direction);
      this.addAnnotation(element, "BBSignal", Collections.<String, Object>unmodifiableMap(CollectionLiterals.<String, Object>newHashMap(_mappedTo)));
      _xblockexpression = element;
    }
    return _xblockexpression;
  }
  
  protected NamedElement _preprocess(final BBDataType element) {
    BBDataType _xblockexpression = null;
    {
      this.addAnnotation(element, "BBDataType", Collections.<String, Object>unmodifiableMap(CollectionLiterals.<String, Object>newHashMap()));
      _xblockexpression = element;
    }
    return _xblockexpression;
  }
  
  protected NamedElement _preprocess(final BBProperty element) {
    BBProperty _xblockexpression = null;
    {
      boolean _isBigEndian = element.isBigEndian();
      Pair<String, Boolean> _mappedTo = Pair.<String, Boolean>of("bigEndian", Boolean.valueOf(_isBigEndian));
      this.addAnnotation(element, "BBProperty", 
        Collections.<String, Object>unmodifiableMap(CollectionLiterals.<String, Object>newHashMap(_mappedTo)));
      _xblockexpression = element;
    }
    return _xblockexpression;
  }
  
  protected NamedElement addAnnotation(final NamedElement element, final String annotationString, final Map<String, Object> parameters) {
    NamedElement _xblockexpression = null;
    {
      Annotation _createAnnotation = CommonFactory.eINSTANCE.createAnnotation();
      final Procedure1<Annotation> _function = (Annotation it) -> {
        it.setName(annotationString);
      };
      final Annotation newAnnotation = ObjectExtensions.<Annotation>operator_doubleArrow(_createAnnotation, _function);
      final BiConsumer<String, Object> _function_1 = (String key, Object paramval) -> {
        EList<AnnotationParameter> _parameters = newAnnotation.getParameters();
        AnnotationParameter _createAnnotationParameter = CommonFactory.eINSTANCE.createAnnotationParameter();
        final Procedure1<AnnotationParameter> _function_2 = (AnnotationParameter it) -> {
          it.setName(key);
          Object _transformBBStereotypePropertyValue = this.transformBBStereotypePropertyValue(paramval);
          it.setValue(_transformBBStereotypePropertyValue);
        };
        AnnotationParameter _doubleArrow = ObjectExtensions.<AnnotationParameter>operator_doubleArrow(_createAnnotationParameter, _function_2);
        _parameters.add(_doubleArrow);
      };
      parameters.forEach(_function_1);
      EList<Annotation> _annotations = element.getAnnotations();
      _annotations.add(newAnnotation);
      _xblockexpression = element;
    }
    return _xblockexpression;
  }
  
  protected Object _extendedTransform(final NamedElement element, final org.eclipse.uml2.uml.NamedElement umlElement) {
    return null;
  }
  
  protected Object _extendedTransform(final BBModel element, final org.eclipse.uml2.uml.Package umlElement) {
    EList<SubSystem> _topSubsystems = null;
    if (element!=null) {
      _topSubsystems=element.getTopSubsystems();
    }
    final List<PackageableElement> transformedTopSubsystems = this.<PackageableElement>transformList(_topSubsystems, 
      PackageableElement.class);
    EList<PackageableElement> _packagedElements = null;
    if (umlElement!=null) {
      _packagedElements=umlElement.getPackagedElements();
    }
    GeneralUtil.<PackageableElement>addAllNonPresent(_packagedElements, transformedTopSubsystems);
    return null;
  }
  
  protected Object _extendedTransform(final SubSystem element, final org.eclipse.uml2.uml.Package umlElement) {
    EList<SubSystem> _subsystems = null;
    if (element!=null) {
      _subsystems=element.getSubsystems();
    }
    final List<PackageableElement> transformedSubsystems = this.<PackageableElement>transformList(_subsystems, 
      PackageableElement.class);
    EList<BBIF> _bbif = null;
    if (element!=null) {
      _bbif=element.getBbif();
    }
    final List<PackageableElement> transformedBBIFs = this.<PackageableElement>transformList(_bbif, PackageableElement.class);
    EList<PackageableElement> _packagedElements = null;
    if (umlElement!=null) {
      _packagedElements=umlElement.getPackagedElements();
    }
    GeneralUtil.<PackageableElement>addAllNonPresent(_packagedElements, transformedSubsystems);
    EList<PackageableElement> _packagedElements_1 = null;
    if (umlElement!=null) {
      _packagedElements_1=umlElement.getPackagedElements();
    }
    GeneralUtil.<PackageableElement>addAllNonPresent(_packagedElements_1, transformedBBIFs);
    return null;
  }
  
  protected Object _extendedTransform(final BBIF element, final org.eclipse.uml2.uml.Package umlElement) {
    EList<BBProtocol> _bbprotocols = null;
    if (element!=null) {
      _bbprotocols=element.getBbprotocols();
    }
    List<PackageableElement> _transformList = this.<PackageableElement>transformList(_bbprotocols, 
      PackageableElement.class);
    final Function1<PackageableElement, org.eclipse.uml2.uml.Package> _function = (PackageableElement it) -> {
      return ProtocolContainerUtils.getProtocolContainer(it);
    };
    final List<org.eclipse.uml2.uml.Package> transformedProtocols = ListExtensions.<PackageableElement, org.eclipse.uml2.uml.Package>map(_transformList, _function);
    EList<BBDataType> _bbdatatypes = null;
    if (element!=null) {
      _bbdatatypes=element.getBbdatatypes();
    }
    final List<PackageableElement> transformedDataTypes = this.<PackageableElement>transformList(_bbdatatypes, 
      PackageableElement.class);
    EList<PackageableElement> _packagedElements = null;
    if (umlElement!=null) {
      _packagedElements=umlElement.getPackagedElements();
    }
    GeneralUtil.<PackageableElement>addAllNonPresent(_packagedElements, transformedProtocols);
    EList<PackageableElement> _packagedElements_1 = null;
    if (umlElement!=null) {
      _packagedElements_1=umlElement.getPackagedElements();
    }
    GeneralUtil.<PackageableElement>addAllNonPresent(_packagedElements_1, transformedDataTypes);
    return null;
  }
  
  protected Object _extendedTransform(final BBProtocol element, final Collaboration umlElement) {
    EList<BBSignal> _bbsignals = null;
    if (element!=null) {
      _bbsignals=element.getBbsignals();
    }
    final List<PackageableElement> transformedSignals = this.<PackageableElement>transformList(_bbsignals, 
      PackageableElement.class);
    return null;
  }
  
  protected Object _extendedTransform(final BBSignal element, final CallEvent umlElement) {
    return null;
  }
  
  protected Object _extendedTransform(final BBDataType element, final org.eclipse.uml2.uml.Class umlElement) {
    EList<BBProperty> _bbproperties = null;
    if (element!=null) {
      _bbproperties=element.getBbproperties();
    }
    final List<Property> transformedProperties = this.<Property>transformList(_bbproperties, 
      Property.class);
    EList<Property> _ownedAttributes = null;
    if (umlElement!=null) {
      _ownedAttributes=umlElement.getOwnedAttributes();
    }
    GeneralUtil.<Property>addAllNonPresent(_ownedAttributes, transformedProperties);
    return null;
  }
  
  protected Object _extendedTransform(final BBProperty element, final org.eclipse.uml2.uml.Package umlElement) {
    return null;
  }
  
  protected Object _transformBBStereotypePropertyValue(final ProtocolBehaviourFeatureKind kind) {
    Direction _switchResult = null;
    if (kind != null) {
      switch (kind) {
        case IN:
          _switchResult = Direction.IN;
          break;
        case OUT:
          _switchResult = Direction.OUT;
          break;
        case INOUT:
          _switchResult = Direction.INOUT;
          break;
        default:
          break;
      }
    }
    return _switchResult;
  }
  
  protected Object _transformBBStereotypePropertyValue(final Object value) {
    return value;
  }
  
  protected <T extends Object> List<T> transformList(final List<? extends EObject> sourceList, final Class<T> type) {
    final Function1<EObject, EObject> _function = (EObject it) -> {
      return this.transform(it);
    };
    List<EObject> _map = ListExtensions.map(sourceList, _function);
    Iterable<T> _filter = Iterables.<T>filter(_map, type);
    return IterableExtensions.<T>toList(_filter);
  }
  
  protected NamedElement preprocess(final NamedElement element) {
    if (element instanceof BBDataType) {
      return _preprocess((BBDataType)element);
    } else if (element instanceof BBModel) {
      return _preprocess((BBModel)element);
    } else if (element instanceof BBProperty) {
      return _preprocess((BBProperty)element);
    } else if (element instanceof BBProtocol) {
      return _preprocess((BBProtocol)element);
    } else if (element instanceof BBSignal) {
      return _preprocess((BBSignal)element);
    } else if (element instanceof BBIF) {
      return _preprocess((BBIF)element);
    } else if (element instanceof SubSystem) {
      return _preprocess((SubSystem)element);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(element).toString());
    }
  }
  
  protected Object extendedTransform(final NamedElement element, final org.eclipse.uml2.uml.NamedElement umlElement) {
    if (element instanceof BBDataType
         && umlElement instanceof org.eclipse.uml2.uml.Class) {
      return _extendedTransform((BBDataType)element, (org.eclipse.uml2.uml.Class)umlElement);
    } else if (element instanceof BBProtocol
         && umlElement instanceof Collaboration) {
      return _extendedTransform((BBProtocol)element, (Collaboration)umlElement);
    } else if (element instanceof BBSignal
         && umlElement instanceof CallEvent) {
      return _extendedTransform((BBSignal)element, (CallEvent)umlElement);
    } else if (element instanceof BBModel
         && umlElement instanceof org.eclipse.uml2.uml.Package) {
      return _extendedTransform((BBModel)element, (org.eclipse.uml2.uml.Package)umlElement);
    } else if (element instanceof BBProperty
         && umlElement instanceof org.eclipse.uml2.uml.Package) {
      return _extendedTransform((BBProperty)element, (org.eclipse.uml2.uml.Package)umlElement);
    } else if (element instanceof BBIF
         && umlElement instanceof org.eclipse.uml2.uml.Package) {
      return _extendedTransform((BBIF)element, (org.eclipse.uml2.uml.Package)umlElement);
    } else if (element instanceof SubSystem
         && umlElement instanceof org.eclipse.uml2.uml.Package) {
      return _extendedTransform((SubSystem)element, (org.eclipse.uml2.uml.Package)umlElement);
    } else if (element != null
         && umlElement != null) {
      return _extendedTransform(element, umlElement);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(element, umlElement).toString());
    }
  }
  
  protected Object transformBBStereotypePropertyValue(final Object kind) {
    if (kind instanceof ProtocolBehaviourFeatureKind) {
      return _transformBBStereotypePropertyValue((ProtocolBehaviourFeatureKind)kind);
    } else if (kind != null) {
      return _transformBBStereotypePropertyValue(kind);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(kind).toString());
    }
  }
}
