package org.eclipse.papyrusrt.bb.bbstruct.papyrus.trans.to.uml;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.papyrusrt.bb.bbIF.BBDataType;
import org.eclipse.papyrusrt.bb.bbIF.BBIF;
import org.eclipse.papyrusrt.bb.bbIF.BBModel;
import org.eclipse.papyrusrt.bb.bbIF.BBProperty;
import org.eclipse.papyrusrt.bb.bbIF.BBProtocol;
import org.eclipse.papyrusrt.bb.bbIF.BBSignal;
import org.eclipse.papyrusrt.bb.bbIF.SubSystem;
import org.eclipse.papyrusrt.bb.bbif.papyrus.trans.to.uml.BBIF2UMLTranslator;
import org.eclipse.papyrusrt.bb.bbstruct.BBBehaviouralCapsule;
import org.eclipse.papyrusrt.bb.bbstruct.BBStructuralCapsule;
import org.eclipse.papyrusrt.xtumlrt.common.Capsule;
import org.eclipse.papyrusrt.xtumlrt.common.NamedElement;
import org.eclipse.papyrusrt.xtumlrt.umlrt.RTModel;
import org.eclipse.papyrusrt.xtumlrt.umlrt.RTPassiveClass;
import org.eclipse.papyrusrt.xtumlrt.util.XTUMLRTLogger;
import org.eclipse.uml2.uml.CallEvent;
import org.eclipse.uml2.uml.Collaboration;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;

@SuppressWarnings("all")
public class BBStruct2UMLTranslator extends BBIF2UMLTranslator {
  @Override
  public int generate(final RTModel model) {
    int _xblockexpression = (int) 0;
    {
      XTUMLRTLogger.info("Invoking BBStruct2UMLTranslator");
      _xblockexpression = super.generate(model);
    }
    return _xblockexpression;
  }
  
  @Override
  protected EObject _transform(final Capsule element) {
    final ArrayList<?> _cacheKey = CollectionLiterals.newArrayList(element);
    final EObject _result;
    synchronized (_createCache__transform) {
      if (_createCache__transform.containsKey(_cacheKey)) {
        return _createCache__transform.get(_cacheKey);
      }
      NamedElement _preprocess = this.preprocess(element);
      EObject __transform = super._transform(((Capsule) _preprocess));
      _result = __transform;
      _createCache__transform.put(_cacheKey, _result);
    }
    _init__transform(_result, element);
    return _result;
  }
  
  private final HashMap<ArrayList<?>, EObject> _createCache__transform = CollectionLiterals.newHashMap();
  
  private void _init__transform(final EObject umlElement, final Capsule element) {
    this.extendedTransform(((Capsule) element), 
      ((org.eclipse.uml2.uml.Class) umlElement));
  }
  
  @Override
  protected EObject _transform(final RTPassiveClass element) {
    final ArrayList<?> _cacheKey = CollectionLiterals.newArrayList(element);
    final EObject _result;
    synchronized (_createCache__transform_1) {
      if (_createCache__transform_1.containsKey(_cacheKey)) {
        return _createCache__transform_1.get(_cacheKey);
      }
      NamedElement _preprocess = this.preprocess(element);
      EObject __transform = super._transform(((RTPassiveClass) _preprocess));
      _result = __transform;
      _createCache__transform_1.put(_cacheKey, _result);
    }
    _init__transform_1(_result, element);
    return _result;
  }
  
  private final HashMap<ArrayList<?>, EObject> _createCache__transform_1 = CollectionLiterals.newHashMap();
  
  private void _init__transform_1(final EObject umlElement, final RTPassiveClass element) {
    this.extendedTransform(((RTPassiveClass) element), 
      ((org.eclipse.uml2.uml.Class) umlElement));
  }
  
  protected NamedElement _preprocess(final BBStructuralCapsule element) {
    BBStructuralCapsule _xblockexpression = null;
    {
      this.addAnnotation(element, "bbStructureCapsule", Collections.<String, Object>unmodifiableMap(CollectionLiterals.<String, Object>newHashMap()));
      _xblockexpression = element;
    }
    return _xblockexpression;
  }
  
  protected NamedElement _preprocess(final BBBehaviouralCapsule element) {
    BBBehaviouralCapsule _xblockexpression = null;
    {
      this.addAnnotation(element, "bbBehaviourCapsule", Collections.<String, Object>unmodifiableMap(CollectionLiterals.<String, Object>newHashMap()));
      _xblockexpression = element;
    }
    return _xblockexpression;
  }
  
  protected Object _extendedTransform(final BBStructuralCapsule element, final org.eclipse.uml2.uml.Class umlElement) {
    return null;
  }
  
  protected Object _extendedTransform(final BBBehaviouralCapsule element, final org.eclipse.uml2.uml.Class umlElement) {
    return null;
  }
  
  protected NamedElement preprocess(final NamedElement element) {
    if (element instanceof BBDataType) {
      return _preprocess((BBDataType)element);
    } else if (element instanceof BBBehaviouralCapsule) {
      return _preprocess((BBBehaviouralCapsule)element);
    } else if (element instanceof BBStructuralCapsule) {
      return _preprocess((BBStructuralCapsule)element);
    } else if (element instanceof BBModel) {
      return _preprocess((BBModel)element);
    } else if (element instanceof BBProperty) {
      return _preprocess((BBProperty)element);
    } else if (element instanceof BBProtocol) {
      return _preprocess((BBProtocol)element);
    } else if (element instanceof BBSignal) {
      return _preprocess((BBSignal)element);
    } else if (element instanceof BBIF) {
      return _preprocess((BBIF)element);
    } else if (element instanceof SubSystem) {
      return _preprocess((SubSystem)element);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(element).toString());
    }
  }
  
  protected Object extendedTransform(final NamedElement element, final org.eclipse.uml2.uml.NamedElement umlElement) {
    if (element instanceof BBDataType
         && umlElement instanceof org.eclipse.uml2.uml.Class) {
      return _extendedTransform((BBDataType)element, (org.eclipse.uml2.uml.Class)umlElement);
    } else if (element instanceof BBBehaviouralCapsule
         && umlElement instanceof org.eclipse.uml2.uml.Class) {
      return _extendedTransform((BBBehaviouralCapsule)element, (org.eclipse.uml2.uml.Class)umlElement);
    } else if (element instanceof BBStructuralCapsule
         && umlElement instanceof org.eclipse.uml2.uml.Class) {
      return _extendedTransform((BBStructuralCapsule)element, (org.eclipse.uml2.uml.Class)umlElement);
    } else if (element instanceof BBProtocol
         && umlElement instanceof Collaboration) {
      return _extendedTransform((BBProtocol)element, (Collaboration)umlElement);
    } else if (element instanceof BBSignal
         && umlElement instanceof CallEvent) {
      return _extendedTransform((BBSignal)element, (CallEvent)umlElement);
    } else if (element instanceof BBModel
         && umlElement instanceof org.eclipse.uml2.uml.Package) {
      return _extendedTransform((BBModel)element, (org.eclipse.uml2.uml.Package)umlElement);
    } else if (element instanceof BBProperty
         && umlElement instanceof org.eclipse.uml2.uml.Package) {
      return _extendedTransform((BBProperty)element, (org.eclipse.uml2.uml.Package)umlElement);
    } else if (element instanceof BBIF
         && umlElement instanceof org.eclipse.uml2.uml.Package) {
      return _extendedTransform((BBIF)element, (org.eclipse.uml2.uml.Package)umlElement);
    } else if (element instanceof SubSystem
         && umlElement instanceof org.eclipse.uml2.uml.Package) {
      return _extendedTransform((SubSystem)element, (org.eclipse.uml2.uml.Package)umlElement);
    } else if (element != null
         && umlElement != null) {
      return _extendedTransform(element, umlElement);
    } else {
      throw new IllegalArgumentException("Unhandled parameter types: " +
        Arrays.<Object>asList(element, umlElement).toString());
    }
  }
}
